#include "filegraphutils.h"


int graphsize(char *s)
/* Get size of graph out of graph6 or sparse6 string. */
{
        char *p;
        int n;

        if (s[0] == ':') p = s+1;
        else             p = s;
        n = *p++ - BIAS6;

        if (n > SMALLN)
        {
            n = *p++ - BIAS6;
            n = (n << 6) | (*p++ - BIAS6);
            n = (n << 6) | (*p++ - BIAS6);
        }
        return n;
}

char * getGraphLine(FILE *f)     /* read a line with error checking */
/* includes \n (if present) and \0.  Immediate EOF causes NULL return. */
{

        int c;
        long i;
        char s [100000];

        i = 0;
        while ((c = getc(f)) != EOF && c != '\n')
        {
            if (i==0 && c=='>'){

                 c=getc(f);
                 c=getc(f);
                 if (c=='g')
                     for(int p = 0;p<8;p++)
                         c=getc(f);
                 else
                     for(int p = 0;p<9;p++)
                         c=getc(f);
             }

             s[i++] = c;

        }

        if (i == 0 && c == EOF) return NULL;

        if (c == '\n') s[i++] = '\n';
        s[i] = '\0';

        return s;
}

void stringToMultigraph(char *s, multigraph *g)
/* Convert string (graph6 or sparse6 format) to Multigraph object. */
/* Assumes g is big enough to hold it. */
{
        char *p;
        int n,i,j,k,v,x,nb;

        n = graphsize(s);
        //g->setNumVertex(n);


        p = s + 1 + (s[0] == ':');

        if (n > SMALLN) p += 3;

        if (s[0] != ':')       /* graph6 format */
        {
            k = 1;
            for (j = 1; j < n; ++j)
            {


                for (i = 0; i < j; ++i)
                {
                    if (--k == 0)
                    {
                        k = 6;
                        x = *(p++) - BIAS6;
                    }

                    if (x & TOPBIT6)
                    {
                        g->setEdge(i,j);

                    }
                    x <<= 1;
                }
            }
        }
        else    /* sparse6 format */
        {
            for (i = n-1, nb = 0; i != 0 ; i >>= 1, ++nb)
            {}

            k = 1;
            v = 0;
            for (;;)
            {
                if (--k == 0)
                {
                    k = 6;
                    if (*p == '\n' || *p == '\0') break;
                    else x = *p - BIAS6;
                    ++p;
                }
                else
                    x <<= 1;

                if (x & TOPBIT6) ++v;
                j = 0;
                for (i = 0; i < nb; ++i)
                {
                    if (--k == 0)
                    {
                        k = 6;
                        if (*p == '\n' || *p == '\0') break;
                        else x = *p - BIAS6;
                        ++p;
                    }
                    else
                        x <<= 1;
                    if (x & TOPBIT6) j = (j << 1) | 1;
                    else             j <<= 1;
                }
                if (i < nb) break;
                if (j > v){
                    v = j;
                }
                else if (v < n)
                {
                    g->setEdge(v,j);
                }
            }
        }
}
FILE* openFileGraph(char * filename){
    FILE * f;
    char c,firstc;
    int codetype;
    bool badHeader;
    if(filename!=NULL){
        f = fopen(filename,"r");
        if(!f)
            return NULL;
    }
    else
        f=stdin;

    firstc = c = getc(f);
        if (c == EOF)
        {
            return f;
        }
    if (c != '>')
        {
            ungetc(c,f);
        }
    else
        {
            badHeader = false;
            if ((c = getc(f)) == EOF || c != '>')
                badHeader = true;
            if (!badHeader && ((c = getc(f)) == EOF || c != 'g' && c != 's'))
                badHeader = true;
            if (!badHeader && c == 'g')
            {
                if ((c = getc(f)) == EOF || c != 'r' ||
                    (c = getc(f)) == EOF || c != 'a' ||
                    (c = getc(f)) == EOF || c != 'p' ||
                    (c = getc(f)) == EOF || c != 'h' ||
                    (c = getc(f)) == EOF || c != '6' ||
                    (c = getc(f)) == EOF || c != '<' ||
                    (c = getc(f)) == EOF || c != '<')
                        badHeader = true;
                else
                    codetype = GRAPH6;
            }
            else if (!badHeader && c == 's')
            {
                if ((c = getc(f)) == EOF || c != 'p' ||
                    (c = getc(f)) == EOF || c != 'a' ||
                    (c = getc(f)) == EOF || c != 'r' ||
                    (c = getc(f)) == EOF || c != 's' ||
                    (c = getc(f)) == EOF || c != 'e' ||
                    (c = getc(f)) == EOF || c != '6' ||
                    (c = getc(f)) == EOF || c != '<' ||
                    (c = getc(f)) == EOF || c != '<')
                        badHeader = true;
                else
                    codetype = SPARSE6;
            }
            if (badHeader)
            {
                fprintf(stderr,">E opengraphfile: illegal header in %s\n",
                        filename == NULL ? "stdin" : filename);

                return NULL;
            }
        }

    return f;
}

void writeGraphLine(FILE *f, char *s)
/* write a line with error checking */
/* \n is not appended automatically */
{
        if (fputs(s,f) == EOF || ferror(f))
        {
            fprintf(stderr,">E writeline : error on writing\n");
        }
}
